#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mathlib.h"
#include "sph_cap.h"
#include "calcChar.h"
#include "ACHAIMDB.h"
#include "ACHAIM_Chars.h"

//****DEBUGGING************
#ifdef DBG
#include "memwatch.h"
#endif
//*************************

int main ()
{
	//declare variables
	FILE *file;
	float latInt, lonInt, altInt;
	double *lat, *lon, *alt;
	double *latout, *lonout, *altout;
	char dbFile[512];
	int nLat; //lat length
	char b[12000]; //fgets buffer
	
	char names[7][5] = {"nmf2","hmf2","hbot","htop","hf1","he","hmf1"}; //names of all outputs
	char inputFile[32] = "ACHAIM.in"; //input file name
	
	file = fopen(inputFile, "r");
	
	if (file == NULL)
	{
		printf("Input file not found, check that your input file path is set properly \n");
		printf("in the input file.\n");
		
		return 0;
	}
	
	//iterate through the header
	while (!feof(file))
	{
		fgets(b, sizeof(b), file);	
		//break when the header is completed
		if (b[0] != '#') break;
	}
	
	//check if full line was read by checking for the new line character
	char *lc = strchr(b,'\n');
	
	if (lc == NULL)
	{	
		printf("Error: Entire latitude line was not read. The line may exceed the 12000 character limit. Unexpected results may occur.\n");
	}
	
	//get lats
	//read the line and parse the data
	//this is taking the line that was previously determined to not be part of the header
	char *p = strtok(b, " ");
	int x=0;
	
	//initial allocation of memory
	lat = malloc(sizeof(double));
	
	//while string token exist
	while (p != NULL)
	{		
		//if first token set interval, otherwise populate array
		(x == 0) ? (latInt = atof(p)) : (lat[x-1] = atof(p));
		
		//check for another token
		p = strtok(NULL, " ");
		
		if (p != NULL)
		{
			if (p[0] == '\r' || p[0] == '\n') break;
			
			//increase memory
			if (x != 0) (lat = realloc(lat, sizeof(double) * (x + 1)));
			x++;
		}
	}
	
	//length of the lat array
	nLat = x;
	
	//get lons
	//read the line and parse the data
	fgets(b, sizeof(b), file);
	
	//check if full line was read by checking for the new line character
	lc = strchr(b,'\n');
	
	if (lc == NULL)
	{	
		printf("Error: Entire longitude line was not read. The line may exceed the 12000 character limit. Unexpected results may occur.\n");
	}
	
	p = strtok(b, " ");
	x=0;
	
	//initial allocation of memory
	lon = malloc(sizeof(double));
	
	//while string token exist
	while (p != NULL)
	{		
		//if first token sent interval, otherwise populate array
		(x == 0) ? (lonInt = atof(p)) : (lon[x-1] = atof(p));
		
		//check for another token
		p = strtok(NULL, " ");
		
		if (p != NULL)
		{
			if (p[0] == '\r' || p[0] == '\n') break;
			
			//increase memory
			if (x != 0) (lon = realloc(lon, sizeof(double) * (x + 1)));
			x++;
		}
	}
	
	//if the lat and long lengths are not the same then error out
	if (nLat != x)
	{
		printf("Error, Latitudes and Longitudes should be the same length\n");
		printf("Latitude length = %i\n", nLat);
			printf("Longitude length = %i\n", x);
		return 0;
	}

	//get alts
	//read the line and parse the data
	fgets(b, sizeof(b), file);
	
	//check if full line was read by checking for the new line character
	lc = strchr(b,'\n');
	
	if (lc == NULL)
	{	
		printf("Error: Entire altitude line was not read. The line may exceed the 12000 character limit. Unexpected results may occur.\n");
	}
	
	p = strtok(b, " ");
	x=0;
	
	//initial allocation of memory
	alt = malloc(sizeof(double));
	
	//while string token exist
	while (p != NULL)
	{		
		//if first token sent interval, otherwise populate array
		(x == 0) ? (altInt = atof(p)) : (alt[x-1] = atof(p));
		
		//check for another token
		p = strtok(NULL, " ");
		
		if (p != NULL)
		{
			if (p[0] == '\r' || p[0] == '\n') break;
			
			//increase memory
			if (x != 0) (alt = realloc(alt, sizeof(double) * (x + 1)));
			x++;
		}
	}
	
	//if the lat and long lengths are not the same then error out
	//if alt == 0 then don't error, asking for TEC
	if (nLat != x && alt[0] != 0 && altInt != 0)
	{
		printf("Error, Altitudes are not the same legth as Latitudes and Longitudes.\n");
		printf("Latitude length = %i\n", nLat);
			printf("Altitude length = %i\n", x);
		return 0;
	}
	
	//Get the DB file path
	fgets(b, sizeof(b), file);
	sscanf(b, "%s", dbFile);	
	
	//close the file
	fclose(file);
	
	//create the input arrays
	//Latitude
	if (latInt == 0)
	{
		latout = lat;
	} else
	{
		nLat = round((lat[1] - lat[0])/latInt)+1;
		latout = calloc(nLat, sizeof(double));
		
		for (int i=0; i<nLat; i++)
			latout[i] = lat[0] + (i * latInt);
	}
	
	//Longitude
	if (lonInt == 0)
	{
		lonout = lon;
	} else
	{
		int tll = round((lon[1] - lon[0])/lonInt)+1;
		
		if (tll != nLat)
		{
			printf("Error, latitude and longitude are not the same length\n");
			printf("Latitude length = %i\n", nLat);
			printf("Longitude length = %i\n", tll);
			return 0;
		}
		
		lonout = calloc(nLat, sizeof(double));
		
		for (int i=0; i<nLat; i++)
			lonout[i] = lon[0] + (i * lonInt);
	}
	
	//Altitude
	if (altInt == 0)
	{
		altout = alt;
	} else
	{
		int al = round((alt[1] - alt[0])/altInt)+1;
		
		if (al != nLat)
		{
			printf("Error, altitude and latitude are not the same length\n");
			printf("Latitude length = %i\n", nLat);
			printf("Longitude length = %i\n", al);
			return 0;
		}
		
		altout = calloc(al, sizeof(double));
		
		for (int i=0; i<al; i++)
			altout[i] = alt[0] + (i * altInt);
	}
	
	//Get all characteristics and print then
	file = fopen("ACHAIMChars.out", "w");
	for (int i=0; i<7; i++)
	{
		fprintf(file, "%s: ", names[i]);
		double *output = AC_Char(names[i], lat, lon, nLat, dbFile);
		for (int i=0; i<nLat; i++)
		{
			fprintf(file, "%f ",output[i]);
		}
		fprintf(file, "\n");
		free(output);
	}
	
	fclose(file); //close the output file
	
	//get the density or TEC and print
	file = fopen("ACHAIMDens.out", "w");
	
	if (alt[0] != 0)
	{
		//Density
		fprintf(file, "dens: ");
		double *output = AC_Dens(lat, lon, alt, nLat, dbFile);
		for (int i=0; i<nLat; i++)
		{
			fprintf(file,"%f ",output[i]);
		}
		fprintf(file, "\n");
		free(output);
	}else
	{
		//TEC
		fprintf(file, "TEC: ");
		double *output = AC_TEC(lat, lon, nLat, dbFile);
		for (int i=0; i<nLat; i++)
		{
			fprintf(file,"%f ",output[i]);
		}
		fprintf(file, "\n");
		free(output);
	}
		

	//free allocated memory
	if (latInt == 0) free(lat); else {free(latout); free(lat);}
	if (lonInt ==0) free(lon); else {free(lonout); free(lon);}
	if (altInt ==0) free(alt); else {free(altout); free(alt);}
	
	return 0;
}