#include <stdio.h>
#include <stdlib.h>
#include "sqlite3.h"
#include "ACHAIMDB.h"

//****DEBUGGING************
#ifdef DBG
#include "memwatch.h"
#endif
//*************************

static double *tempCallback; //pointer to the indices to be returned by the functions
int cbc; //callback counter

//declare static function
static int callback(void *data, int argc, char **argv, char **azColName);

//sqlite3 callback function for data
static int callback(void *data, int argc, char **argv, char **azColName)
{   
	(void)data;
	(void)azColName;
	
	if (argv[0] == NULL) {return 0;}
	
	tempCallback[cbc] = atof(argv[0]);
	cbc++;
		
   return 0;
}

//Routine to get the scalar integer values from the DB
//Inputs:
//name = the charcteristic name of interest (char *)
//val = name of value to retreive (char *)
//db = the SQLite DB object (sqlite *)
//Output: DB value (int)
int getScalarDB(char *name, char *val, sqlite3 **db)
{
	//variables
	char sql[128]; //sql command
	char *zErrMsg; //sql error
	int rc; //sqlite3 return code

	//set the output pointer to the global variable
	//this is necessary for the callback function
	tempCallback = calloc(1, sizeof(double));

	//Create the SQL query
	sprintf(sql,"Select %s from Info Where Name LIKE \'%s\'", val, name);

	//set the callback counter to 0
	cbc = 0;
	
	//Execute the query, getting the data
	rc = sqlite3_exec(*db, sql, callback, 0, &zErrMsg);
	if (rc != 0) printf("Error %i in getScalarDB: %s\n%s\n", rc, zErrMsg, sql);
	
	//cleanup and return value
	int ret = (int)tempCallback[0];
	free(tempCallback);
	
	return ret;
	
}

//Routine to get the double vectors from the DB
//Inputs:
//name = the charcteristic name of interest (char *)
//val = name of value to retreive (char *)
//l0 = returns the length of the vector (int)
//db = the SQLite DB object (sqlite *)
//Output: DB vector (double *)
double *getVectorDB(char *name, char *val, int *l0, sqlite3 *db)
{
	//variables
	char sql[128]; //sql command
	char *zErrMsg; //sql error
	int rc; //sqlite3 return code

	//set the output pointer to the global variable
	//this is necessary for the callback function
	tempCallback = calloc(169, sizeof(double)); //set to the max value we expect from these arrays

	//Create the SQL query
	sprintf(sql,"Select %s from %s", name, val);

	//set the callback counter to 0
	cbc = 0;
	
	//Execute the query, getting the data
	rc = sqlite3_exec(db, sql, callback, 0, &zErrMsg);
	if (rc != 0) printf("Error %i in getVectorDB: %s\n%s\n", rc, zErrMsg, sql);
	
	*l0 = cbc;
	
	return tempCallback;
	
}