#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "sph_cap.h"
#include "inGrid.h"

#define DEG2RAD 0.0174532925199432957692369076

//function to calculate if a latitude longitude pair
//is within the assimilation grid
//Inputs:
//rot = the rotation matrix for the grid
//bot = bottom latitude of the grid
//lat = input latitudes
//lon = input longitudes
//nLat = length of lat/lon arrays
//Output: 'logical' array indicating if the pair is within the grid (1) or not (0)
double *inGrid(double *rot, double bot, double *lat, double *lon, int nLat)
{
	//variables
	double *output; //output array
	double *ones, *onesSph, *latSph, *lonSph, *latRad, *lonRad; //arrays for steps of transformed coordinates
	double c[3] = {0,0,0}, norm, h, a, b; //calculation temp variables

	//allocate arrays
	ones = calloc(nLat, sizeof(double));
	latRad = calloc(nLat, sizeof(double));
	lonRad = calloc(nLat, sizeof(double));
	onesSph = calloc(nLat, sizeof(double));
	latSph = calloc(nLat, sizeof(double));
	lonSph = calloc(nLat, sizeof(double));
	output = calloc(nLat, sizeof(double)); 
	
	
	//convert latitude and longitude to radians
	//and populate the ones array
	for (int i=0; i<nLat; i++)
	{
		latRad[i] = lat[i] * DEG2RAD;
		lonRad[i] = lon[i] * DEG2RAD;
		ones[i] = 1;
	}
	
	//convert latitude/longitude to spherical
	cart2sph(ones, latRad, lonRad, nLat, -1, onesSph, latSph, lonSph);
	
	//set up variables for calculation
	c[0] = rot[6] * 17000000; c[1] = rot[7] * 17000000;	c[2] = rot[8] * 17000000;
	norm = sqrt(pow(c[0],2) + pow(c[1],2) + pow(c[2],2));
	c[0] /= norm; c[1] /= norm; c[2] /= norm;
	
	for (int i=0; i<nLat; i++)
	{
		h = (onesSph[i] * c[0]) + (latSph[i] * c[1]) + (lonSph[i] * c[2]);
		a = h * tan((90 - bot) * DEG2RAD);
		b = sqrt(pow(onesSph[i] - (h * c[0]),2) + pow(latSph[i] - (h * c[1]),2) \
				+ pow(lonSph[i] - (h * c[2]),2));
		output[i] = (a > b) ? 1 : 0;
	}
	
	//free memory
	free(onesSph);
	free(latSph);
	free(lonSph);
	free(ones);
	free(lonRad);
	free(latRad);
	
	return output;
}